#
#  FILE NAME:
#      drop.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging

# P3 imports:
from p3.base import safe_fmt
from p3.db.dbms import DBMS
from p3.db.error import DbModuleError
from p3.db.manage.check import db_exists


_LOG = logging.getLogger( "p3.db" )


## \param mng_conn DbConnection object which will be used for database management tasks;
#  \param db_name name of a database to be dropped;
#  \param skip_not_existing flag indicating that the function should attempt database drop only
#      when it exists;
def drop_db( mng_conn, db_name, skip_not_existing=False ):
    _LOG.info( "Dropping '%s' database...", db_name )

    assert isinstance( db_name, str )

    if skip_not_existing and not db_exists( mng_conn, db_name ):
        _LOG.info( "Skipping droping of not existing '%s' database.", db_name )
        return

    if mng_conn.dbms == DBMS.mysql:
        drop_my_sql_db( mng_conn, db_name )
    elif mng_conn.dbms == DBMS.pgsql:
        drop_pg_sql_db( mng_conn, db_name )
    else:
        msg = safe_fmt( "Unsupported database management system: '%s'",
                        '/'.join( DBMS.get_names( mng_conn.dbms ) ) )
        raise DbModuleError( DbModuleError.codes.dbms_not_supported, msg )


def drop_my_sql_db( mng_conn, db_name ):
    query =  "DROP DATABASE `{0:s}`;".format( db_name )
    mng_conn.cursor().execute( query )


def drop_pg_sql_db( mng_conn, db_name ):
    import psycopg2.extensions

    # To understand why set_isolation_level method is invoked, see comment in the
    # 'create_pg_sql_db' function.

    prev_level = mng_conn.db_conn.isolation_level
    mng_conn.db_conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
    query =  "DROP DATABASE \"{0:s}\";".format( db_name )
    mng_conn.cursor().execute( query )
    mng_conn.db_conn.set_isolation_level( prev_level )
