#
#  FILE NAME:
#      int_converter.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging
import numbers

# P3 imports:
from p3.base import Enum
from p3.logic.var.converter.var_converter import VarConverter


_LOG = logging.getLogger( "p3.logic" )


class IntConverter( VarConverter ):
    errors = Enum(
        "empty_input", # The input string is empty.
        "value_error", # The input string couldn't be converted to an integer.
        "out_of_set",  # The deserialization output value or serialization input value is not
                       #  included in the set of acceptable values.
        )

    ## ===[ Initialization ]==================================================================== ##

    ## \param accept_cb A callback returning True when the integer given as its only parameter is
    #         not in the set of acceptable values.
    def __init__( self, accept_cb=None ):
        super( IntConverter, self ).__init__(
            self._deserialize, self._serialize, lambda v: isinstance( v, numbers.Integral ) )

        self._accept_cb = accept_cb

    ## ===[ Conversion ]======================================================================== ##

    def _deserialize( self, res ):
        if not res.in_val:
            res.errors.append( self.errors.empty_input )
            return None

        try:
            out_val = int(res.in_val)
        except ValueError as e:
            res.errors.append( self.errors.value_error )
        else:
            if self._accept_cb is not None and not self._accept_cb( out_val ):
                res.errors.append( self.errors.out_of_set )

        if not res.errors:
            return out_val
        else:
            return None


    def _serialize( self, res ):
        if self._accept_cb is not None and not self._accept_cb( res.in_val ):
            res.errors.append( self.errors.out_of_set )
            return None
        else:
            return str(res.in_val)
